library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

-- ------------------------------------------------------------------
-- Module:  memVar16
-- Purpose: memVar16 has a synchronized 8-bit interface and allows 16 bit wide
--          asynchronous read access.
--          Data update through the 8-bit interface is done in two consecutive
--          write cycles, where the data is updated synchronized to the second
--          write cycle, so the asynchronous read never sees an invalid 
--          intermediate value.
--          Read access through the 8-bit interface needs 2 consecutive read steps too.
--          The inv-line is used to invert the byte order.
-- Author:  myself
-- Created: 20090808
-- ------------------------------------------------------------------
entity memVar16 is
    Port     ( clk      : in  std_logic;  -- system clock
               we       : in  std_logic;  -- write enable (active high)
               re       : in  std_logic;  -- read enable (active high)
               inv      : in  std_logic;  -- invert byte order (active high)
               di       : in  std_logic_vector (7 downto 0);   -- 8-Bit interface input
               do       : out std_logic_vector (7 downto 0);   -- 8-Bit interface output
               asData   : out std_logic_vector(15 downto 0)    -- asynchronous read access
             );
end memVar16;

architecture std of memVar16 is
   signal   data   : std_logic_vector (15 downto 0) := (others=>'0');
   signal   tmp    : std_logic_vector (7 downto 0)  := (others=>'0');
   signal   fWrite : std_logic := '1';
   signal   fRead  : std_logic := '1';
begin
   asData <= data;   -- asynchronous read using full data width

   process
      variable cmd : std_logic_vector (2 downto 0) := (others=>'0');
   begin
      wait until rising_edge(clk);
      cmd := inv & fWrite & fRead;
      -- synchronous 8-bit write with write enable
      if (we = '1') then
         case cmd is
         when "000" | "001"  => data   <= tmp & di;
                                fWrite <= '1';
         when "010" | "011"  => tmp    <= di;
                                fWrite <= '0';
         when "100" | "101"  => data   <= di & tmp;
                                fWrite <= '1';
         when "110" | "111"  => tmp    <= di;
                                fWrite <= '0';
         when others         => NULL;                        
         end case;
      end if;
      -- synchronous 8-bit read with read enable
      if (re = '1') then
         case cmd is
         when "000" | "010"  => do    <= data(15 downto 8);
                                         fRead <= '1';
         when "001" | "011"  => do    <= data(7 downto 0);
                                         fRead <= '0';
         when "100" | "110"  => do    <= data(7 downto 0);
                                         fRead <= '1';
         when "101" | "111"  => do    <= data(15 downto 8);
                                         fRead <= '0';
         when others         => NULL;                       
         end case;
      end if;
   end process;
end std;
